/*
 * config file for xbuffy.  
 */

/* Enable debugging info */
#undef DEBUG

@TOP@

/* if you want Xbuffy to try to use Content-Length: headers uncomment the
 * following 
 */
#undef USE_CONTENT_LENGTH

/* Version Information */
#undef VERSION

/* enable NNTP mailboxes? */
#undef USE_NNTP

/* Default NNTP Server */ 
#undef NNTP_SERVER

/* use Motif instead of Athena? */
#undef USE_MOTIF

/* enable keyboard LED blinking? */
#undef USE_LED
